@@ -1,21 +1,30 @@
 def check_filters(cui, filters):
-    r''' Checks is a CUI in the filters
+    """Checks is a CUI in the filters
 
     Args:
         cui
         filters
 
-    Return:
+    Returns:
         bool:
             True if in filters else False
-    '''
-    if cui in filters.get('cuis', {}) or not filters.get('cuis', {}):
-        return cui not in filters.get('cuis_exclude', {})
+    """
+    if cui in filters.cuis or not filters.cuis:
+        return cui not in filters.cuis_exclude
     else:
         return False
 
 
-def get_project_filters(cuis, type_ids, cdb):
+def get_all_irrelevant_cuis(project, cdb):
+    i_cuis = set()
+    for d in project['documents']:
+        for a in d['annotations']:
+            if 'irrelevant' in a and a['irrelevant']:
+                i_cuis.add(a['cui'])
+    return i_cuis
+
+
+def get_project_filters(cuis, type_ids, cdb, project=None):
     cui_filter = set()
     if isinstance(cuis, str):
         if cuis is not None and cuis:
@@ -35,4 +44,9 @@ def get_project_filters(cuis, type_ids, cdb):
     elif isinstance(cuis, list):
         cui_filter = set(cuis)
 
+    if project is not None:
+        i_cuis = get_all_irrelevant_cuis(project, cdb)
+        for i_cui in i_cuis:
+            cui_filter.remove(i_cui)
+
     return cui_filter
