@@ -3,27 +3,35 @@ from medcat.cdb import CDB
 from medcat.preprocessing.cleaners import clean_name
 from medcat.utils.other import TPL_ENT, TPL_ENTS
 
+import logging
+
+logger = logging.getLogger(__name__)
+
 
 def get_important_config_parameters(config):
     cnf = {
-            "config.ner['min_name_len']": {
-                'value': config.ner['min_name_len'],
+            "config.ner.min_name_len": {
+                'value': config.ner.min_name_len,
                 'description': "Minimum detection length (found terms/mentions shorter than this will not be detected)."
                 },
-            "config.ner['upper_case_limit_len']": {
-                'value': config.ner['upper_case_limit_len'],
+            "config.ner.upper_case_limit_len": {
+                'value': config.ner.upper_case_limit_len,
                 'description': "All detected terms shorter than this value have to be uppercase, otherwise they will be ignored."
                 },
-            "config.linking['similarity_threshold']": {
-                'value': config.linking['similarity_threshold'],
+            "config.linking.similarity_threshold": {
+                'value': config.linking.similarity_threshold,
                 'description': "If the confidence of the model is lower than this a detection will be ignore."
                 },
-            "config.general['spell_check']": {
-                'value': config.general['spell_check'],
+            "config.linking.filters.cuis": {
+                'value': len(config.linking.filters.cuis),
+                'description': "Length of the CUIs filter to be included in outputs. If this is not 0 (i.e. not empty) its best to check what is included before using the model"
+            },
+            "config.general.spell_check": {
+                'value': config.general.spell_check,
                 'description': "Is spell checking enabled."
                 },
-            "config.general['spell_check_len_limit']": {
-                'value': config.general['spell_check_len_limit'],
+            "config.general.spell_check_len_limit": {
+                'value': config.general.spell_check_len_limit,
                 'description': "Words shorter than this will not be spell checked."
                 },
             }
@@ -105,8 +113,7 @@ def json2html(doc):
 
 
 def prepare_name(cat, name, version='CLEAN'):
-    """ Cleans up the name
-    """
+    """Cleans up the name."""
     name = clean_name(name)
 
     if version.lower() == 'clean':
@@ -173,10 +180,14 @@ def tkns_from_doc(spacy_doc, start, end):
 
 
 def filter_cdb_by_icd10(cdb: CDB) -> CDB:
-    """
-    Filters an existing CDB to only contain concepts that have an associated ICD-10 code.
+    """Filters an existing CDB to only contain concepts that have an associated ICD-10 code.
     Can be used for snomed orr UMLS CDBs.
-    :return: filtered CDB
+
+    Args:
+        CDB: The input CDB
+
+    Returns:
+        CDB: The filtered CDB
     """
     cuis_to_keep = [cui for cui in cdb.cui2names.keys() if 'icd10' in cdb.cui2info[cui]]
     cdb.filter_by_cui(cuis_to_keep)
@@ -206,8 +217,8 @@ def umls_to_icd10cm(cdb, csv_path):
                             cdb.cui2info[cui]['icd10'].append(icd10)
                     else:
                         cdb.cui2info[cui]["icd10"] = [icd10]
-        except Exception:
-            print(row["CUI"])
+        except Exception as e:
+            logger.warn("Issue at %s", row["CUI"], exc_info=e)
 
 
 def umls_to_icd10_over_snomed(cdb, pickle_path):
@@ -224,7 +235,7 @@ def umls_to_icd10_over_snomed(cdb, pickle_path):
                     # If it exists skip it
                     pass
                 else:
-                    print(cui, icd10)
+                    logger.info("%s %s", cui, icd10)
                     cdb.cui2info[cui]['icd10'] = [icd10]
             else:
                 pass
@@ -239,7 +250,7 @@ def umls_to_icd10_ext(cdb, pickle_path):
             if cui in cdb.cui2info and 'icd10' not in cdb.cui2info[cui]:
                 icd10 = u2i[cui]
 
-                print(cui, icd10)
+                logger.info("%s %s", cui, icd10)
                 cdb.cui2info[cui]['icd10'] = [icd10]
 
 
@@ -266,13 +277,12 @@ def umls_to_icd10(cdb, csv_path):
                         cdb.cui2info[cui]['icd10'].append(icd10)
                 else:
                     cdb.cui2info[cui]["icd10"] = [icd10]
-        except Exception:
-            print(row["CUI"])
+        except Exception as e:
+            logger.warn("Issue at %s", row["CUI"], exc_info=e)
 
 
 def umls_to_snomed(cdb, pickle_path):
-    """ Map UMLS CDB to SNOMED concepts
-    """
+    """Map UMLS CDB to SNOMED concepts."""
     import pickle
 
     data = pickle.load(open(pickle_path, 'rb'))
@@ -293,8 +303,7 @@ def umls_to_snomed(cdb, pickle_path):
 
 
 def snomed_to_umls(cdb, pickle_path):
-    """ Map SNOMED CDB to UMLS concepts
-    """
+    """Map SNOMED CDB to UMLS concepts."""
     import pickle
 
     data = pickle.load(open(pickle_path, 'rb'))
@@ -315,8 +324,7 @@ def snomed_to_umls(cdb, pickle_path):
 
 
 def snomed_to_icd10(cdb, csv_path):
-    """ Add map from cui to icd10 for concepts
-    """
+    """Add map from cui to icd10 for concepts."""
     import pandas as pd
     df = pd.read_csv(csv_path)
 
@@ -340,8 +348,7 @@ def snomed_to_icd10(cdb, csv_path):
 
 
 def snomed_to_desc(cdb, csv_path):
-    """ Add descriptions to the concepts
-    """
+    """Add descriptions to the concepts."""
     import pandas as pd
     df = pd.read_csv(csv_path)
 
@@ -382,11 +389,11 @@ def add_names_icd10(csv_path, cat):
             cui = str(row['cui'])
             name = row['name']
             cat.add_name(cui, name, is_pref_name=False, only_new=True)
-        except Exception:
-            print(row["cui"])
+        except Exception as e:
+            logger.warn("Issue at %s", row["CUI"], exc_info=e)
 
         if index % 1000 == 0:
-            print(index)
+            logger.info('index=%d', index)
 
 
 def add_names_icd10cm(cdb, csv_path, cat):
@@ -401,13 +408,13 @@ def add_names_icd10cm(cdb, csv_path, cat):
                 bl = len(cdb.cui2names.get(cui, []))
                 cat.add_name(cui, name, is_pref_name=False, only_new=True)
                 if bl != len(cdb.cui2names.get(cui, [])):
-                    print(name, cui)
+                    logger.info("'%s' with cui '%s'", name, cui)
         except Exception as e:
-            print(e)
+            logger.warn("Issue at %s", row["CUI"], exc_info=e)
             break
 
         if index % 1000 == 0:
-            print(index)
+            logger.info("Index=%d", index)
 
 
 def remove_icd10_ranges(cdb):
@@ -431,7 +438,7 @@ def dep_check_scispacy():
     try:
         _ = spacy.load("en_core_sci_md")
     except Exception:
-        print("Installing the missing models for scispacy\n")
+        logger.info("Installing the missing models for scispacy\n")
         pkg = 'https://s3-us-west-2.amazonaws.com/ai2-s2-scispacy/releases/v0.4.0/en_core_sci_md-0.4.0.tar.gz'
         subprocess.check_call([sys.executable, '-m', 'pip', 'install', pkg])
 
