@@ -9,29 +9,35 @@ from torch import nn
 from scipy.special import softmax
 from medcat.config_meta_cat import ConfigMetaCAT
 from medcat.tokenizers.meta_cat_tokenizers import TokenizerWrapperBase
-from sklearn.metrics import classification_report, f1_score, precision_recall_fscore_support
+from sklearn.metrics import classification_report, precision_recall_fscore_support
+
+import logging
+
+
+logger = logging.getLogger(__name__)
 
 
 def set_all_seeds(seed: int) -> None:
     torch.manual_seed(seed)
     np.random.seed(seed)
+    random.seed(seed)
 
 
 def create_batch_piped_data(data: List, start_ind: int, end_ind: int, device: torch.device, pad_id: int) -> Tuple:
-    r''' Creates a batch given data and start/end that denote batch size, will also add
+    """Creates a batch given data and start/end that denote batch size, will also add
     padding and move to the right device.
 
     Args:
         data (List[List[int], int, Optional[int]]):
             Data in the format: [[<[input_ids]>, <cpos>, Optional[int]], ...], the third column is optional
             and represents the output label
-        start_ind (`int`):
+        start_ind (int):
             Start index of this batch
-        end_ind (`int`):
+        end_ind (int):
             End index of this batch
-        device (`torch.device`):
+        device (torch.device):
             Where to move the data
-        pad_id (`int`):
+        pad_id (int):
             Padding index
 
     Returns:
@@ -39,8 +45,7 @@ def create_batch_piped_data(data: List, start_ind: int, end_ind: int, device: to
             Same as data, but subsetted and as a tensor
         cpos ():
             Center positions for the data
-
-    '''
+    """
     max_seq_len = max([len(x[0]) for x in data])
     x = [x[0][0:max_seq_len] + [pad_id]*max(0, max_seq_len - len(x[0])) for x in data[start_ind:end_ind]]
     cpos = [x[1] for x in data[start_ind:end_ind]]
@@ -56,7 +61,7 @@ def create_batch_piped_data(data: List, start_ind: int, end_ind: int, device: to
 
 
 def predict(model: nn.Module, data: List, config: ConfigMetaCAT) -> Tuple:
-    r''' Predict on data used in the meta_cat.pipe
+    """Predict on data used in the meta_cat.pipe
 
     Args:
         data (List[List[List[int], int]]):
@@ -69,7 +74,7 @@ def predict(model: nn.Module, data: List, config: ConfigMetaCAT) -> Tuple:
             For each row of input data a prediction
         confidence (List[float]):
             For each prediction a confidence value
-    '''
+    """
 
     pad_id = config.model['padding_idx']
     batch_size = config.general['batch_size_eval']
@@ -101,13 +106,13 @@ def predict(model: nn.Module, data: List, config: ConfigMetaCAT) -> Tuple:
 
 
 def split_list_train_test(data: List, test_size: int, shuffle: bool = True) -> Tuple:
-    r''' Shuffle and randomply split data
+    """Shuffle and randomply split data
 
     Args:
         data
         test_size
         shuffle
-    '''
+    """
     if shuffle:
         random.shuffle(data)
 
@@ -119,7 +124,7 @@ def split_list_train_test(data: List, test_size: int, shuffle: bool = True) -> T
 
 
 def print_report(epoch: int, running_loss: List, all_logits: List, y: Any, name: str = 'Train') -> None:
-    r''' Prints some basic stats during training
+    """Prints some basic stats during training
 
     Args:
         epoch
@@ -127,20 +132,20 @@ def print_report(epoch: int, running_loss: List, all_logits: List, y: Any, name:
         all_logits
         y
         name
-    '''
+    """
     if all_logits:
-        print(f'Epoch: {epoch} ' + "*"*50 + f"  {name}")
-        print(classification_report(y, np.argmax(np.concatenate(all_logits, axis=0), axis=1)))
+        logger.info('Epoch: %d %s %s', epoch, "*"*50, name)
+        logger.info(classification_report(y, np.argmax(np.concatenate(all_logits, axis=0), axis=1)))
 
 
 def train_model(model: nn.Module, data: List, config: ConfigMetaCAT, save_dir_path: Optional[str] = None) -> Dict:
-    r''' Trains a LSTM model (for now) with autocheckpoints
+    """Trains a LSTM model (for now) with autocheckpoints
 
     Args:
         data
         config
         save_dir_path
-    '''
+    """
     # Get train/test from data
     train_data, test_data = split_list_train_test(data, test_size=config.train['test_size'], shuffle=config.train['shuffle_data'])
     device = torch.device(config.general['device']) # Create a torch device
@@ -167,7 +172,7 @@ def train_model(model: nn.Module, data: List, config: ConfigMetaCAT, save_dir_pa
     y_test = [x[2] for x in test_data]
     y_train = [x[2] for x in train_data]
 
-    winner_report= {'f1': 0, 'report': '', 'epoch': 0}
+    winner_report: Dict = {}
     for epoch in range(nepochs):
         running_loss = []
         all_logits = []
@@ -201,11 +206,11 @@ def train_model(model: nn.Module, data: List, config: ConfigMetaCAT, save_dir_pa
         print_report(epoch, running_loss, all_logits, y=y_train, name='Train')
         print_report(epoch, running_loss_test, all_logits_test, y=y_test, name='Test')
 
-        score_average = config.train['score_average']
-        f1 = f1_score(y_test, np.argmax(np.concatenate(all_logits_test, axis=0), axis=1), average=score_average)
-        if f1 > winner_report['f1']:
+        _report = classification_report(y_test, np.argmax(np.concatenate(all_logits_test, axis=0), axis=1), output_dict=True)
+        if not winner_report or _report[config.train['metric']['base']][config.train['metric']['score']] > \
+                winner_report['report'][config.train['metric']['base']][config.train['metric']['score']]:
+
             report = classification_report(y_test, np.argmax(np.concatenate(all_logits_test, axis=0), axis=1), output_dict=True)
-            winner_report['f1'] = f1
             winner_report['report'] = report
             winner_report['epoch'] = epoch
 
@@ -217,20 +222,20 @@ def train_model(model: nn.Module, data: List, config: ConfigMetaCAT, save_dir_pa
                 else:
                     path = os.path.join(save_dir_path, 'model.dat')
                     torch.save(model.state_dict(), path)
-                    print("\n##### Model saved to {} at epoch: {} and f1: {} #####\n".format(path, epoch, f1))
+                    logger.info("\n##### Model saved to %s at epoch: %d and %s/%s: %s #####\n", path, epoch, config.train['metric']['base'],
+                          config.train['metric']['score'], winner_report['report'][config.train['metric']['base']][config.train['metric']['score']])
 
     return winner_report
 
 
 def eval_model(model: nn.Module, data: List, config: ConfigMetaCAT, tokenizer: TokenizerWrapperBase) -> Dict:
-    r''' Evaluate a trained model on the provided data
+    """Evaluate a trained model on the provided data
 
     Args:
         model
         data
         config
-
-    '''
+    """
     device = torch.device(config.general['device']) # Create a torch device
     batch_size_eval = config.general['batch_size_eval']
     pad_id = config.model['padding_idx']
@@ -273,6 +278,7 @@ def eval_model(model: nn.Module, data: List, config: ConfigMetaCAT, tokenizer: T
         p = id2category_value[p]
         c = data[i][1]
         tkns = data[i][0]
+        assert tokenizer.hf_tokenizers is not None
         text = tokenizer.hf_tokenizers.decode(tkns[0:c]) + " <<"+ tokenizer.hf_tokenizers.decode(tkns[c:c+1]).strip() + ">> " + \
             tokenizer.hf_tokenizers.decode(tkns[c+1:])
         info = "Predicted: {}, True: {}".format(p, y)
