@@ -12,8 +12,7 @@ def parse_file(filename, first_row_header=True, columns=None):
 
 
 class Snomed:
-    r"""
-    Pre-process SNOMED CT release files:
+    """Pre-process SNOMED CT release files:
     Args:
         data_path:
             Path to the unzipped SNOMED CT folder
@@ -21,14 +20,14 @@ class Snomed:
             Specification of a SNOMED UK extension after 2021 to process the divergent release format.
     """
 
-    def __init__(self, data_path, uk_ext=False):
+    def __init__(self, data_path, uk_ext=False, uk_drug_ext=False):
         self.data_path = data_path
         self.release = data_path[-16:-8]
         self.uk_ext = uk_ext
+        self.uk_drug_ext = uk_drug_ext
 
-    def to_concept_df(self, ):
-        """
-        Please remember to specify if the version is a SNOMED UK extension released after 2021.
+    def to_concept_df(self):
+        """Please remember to specify if the version is a SNOMED UK extension released after 2021.
         This can be done prior to this step: Snomed.uk_ext = True.
         This step is not required for UK extension releases pre-2021.
 
@@ -63,6 +62,17 @@ class Snomed:
                     continue
                 else:
                     pass
+            if self.uk_drug_ext:
+                if "SnomedCT_UKDrugRF2_PRODUCTION" in paths[i]:
+                    concept_snapshot = "sct2_Concept_UKDGSnapshot"
+                    description_snapshot = "sct2_Description_UKDGSnapshot-en"
+                elif "SnomedCT_UKEditionRF2_PRODUCTION" in paths[i]:
+                    concept_snapshot = "sct2_Concept_UKEDSnapshot"
+                    description_snapshot = "sct2_Description_UKEDSnapshot-en"
+                elif "SnomedCT_UKClinicalRefsetsRF2_PRODUCTION" in paths[i]:
+                    continue
+                else:
+                    pass
 
             for f in os.listdir(contents_path):
                 m = re.search(f'{concept_snapshot}'+r'_(.*)_\d*.txt', f)
@@ -111,8 +121,7 @@ class Snomed:
         return pd.concat(df2merge).reset_index(drop=True)
 
     def list_all_relationships(self):
-        """
-        SNOMED CT provides a rich set of inter-relationships between concepts.
+        """SNOMED CT provides a rich set of inter-relationships between concepts.
 
         :return: List of all SNOMED CT relationships
         """
@@ -148,6 +157,17 @@ class Snomed:
                     continue
                 else:
                     pass
+            if self.uk_drug_ext:
+                if "SnomedCT_UKDrugRF2_PRODUCTION" in paths[i]:
+                    concept_snapshot = "sct2_Concept_UKDGSnapshot"
+                    relationship_snapshot = "sct2_Relationship_UKDGSnapshot"
+                elif "SnomedCT_UKEditionRF2_PRODUCTION" in paths[i]:
+                    concept_snapshot = "sct2_Concept_UKEDSnapshot"
+                    relationship_snapshot = "sct2_Relationship_UKEDSnapshot"
+                elif "SnomedCT_UKClinicalRefsetsRF2_PRODUCTION" in paths[i]:
+                    continue
+                else:
+                    pass
 
             for f in os.listdir(contents_path):
                 m = re.search(f'{concept_snapshot}'+r'_(.*)_\d*.txt', f)
@@ -161,9 +181,7 @@ class Snomed:
         return all_rela
 
     def relationship2json(self, relationshipcode, output_jsonfile):
-        """
-
-        :param relationshipcode: A single SCTID or unique concept identifier of the relationship type
+        """:param relationshipcode: A single SCTID or unique concept identifier of the relationship type
         :param output_jsonfile: Name of json file output. Tip: include SNOMED edition to avoid downstream confusions
         :return: json file  of relationship mapping
         """
@@ -199,6 +217,18 @@ class Snomed:
                     continue
                 else:
                     pass
+            if self.uk_drug_ext:
+                if "SnomedCT_UKDrugRF2_PRODUCTION" in paths[i]:
+                    concept_snapshot = "sct2_Concept_UKDGSnapshot"
+                    relationship_snapshot = "sct2_Relationship_UKDGSnapshot"
+                elif "SnomedCT_UKEditionRF2_PRODUCTION" in paths[i]:
+                    concept_snapshot = "sct2_Concept_UKEDSnapshot"
+                    relationship_snapshot = "sct2_Relationship_UKEDSnapshot"
+                elif "SnomedCT_UKClinicalRefsetsRF2_PRODUCTION" in paths[i]:
+                    continue
+                else:
+                    pass
+
             for f in os.listdir(contents_path):
                 m = re.search(f'{concept_snapshot}'+r'_(.*)_\d*.txt', f)
                 if m:
@@ -214,7 +244,8 @@ class Snomed:
                     relationship[_].append(v['sourceId'])
                 else:
                     pass
-            output_dict.update(relationship)
+            output_dict = {key: output_dict.get(key, []) + relationship.get(key, []) for key in
+                           set(list(output_dict.keys()) + list(relationship.keys()))}
         with open(output_jsonfile, 'w') as json_file:
             json.dump(output_dict, json_file)
         return
@@ -251,6 +282,15 @@ class Snomed:
                     continue
                 else:
                     pass
+            if self.uk_drug_ext:
+                if "SnomedCT_UKDrugRF2_PRODUCTION" in paths[i]:
+                    icd10_ref_set = "der2_iisssciRefset_ExtendedMapUKDGSnapshot"
+                elif "SnomedCT_UKEditionRF2_PRODUCTION" in paths[i]:
+                    icd10_ref_set = "der2_iisssciRefset_ExtendedMapUKEDSnapshot"
+                elif "SnomedCT_UKClinicalRefsetsRF2_PRODUCTION" in paths[i]:
+                    continue
+                else:
+                    pass
             for f in os.listdir(refset_terminology):
                 m = re.search(f'{icd10_ref_set}'+r'_(.*)_\d*.txt', f)
                 if m:
@@ -262,10 +302,15 @@ class Snomed:
             df2merge.append(icd_mappings)
         return pd.concat(df2merge)
 
-    def map_snomed2opcs4(self):
-        """
+    def map_snomed2opcs4(self) -> pd.DataFrame:
+        """Map SNOMED to OPSC4.
 
-        :return: SNOMED to OPSC4 mapping DataFrame which includes all metadata
+        Raises:
+            FileNotFoundError: In case of incorrect path to SNOMED CT directory.
+            FileNotFoundError: If OPCS mapping files not found.
+
+        Returns:
+            pd.DataFrame: SNOMED to OPSC4 mapping DataFrame which includes all metadata.
         """
         snomed_releases = []
         paths = []
@@ -295,6 +340,15 @@ class Snomed:
                     continue
                 else:
                     pass
+            if self.uk_drug_ext:
+                if "SnomedCT_UKDrugRF2_PRODUCTION" in paths[i]:
+                    opcs4_ref_set = "der2_iisssciRefset_ExtendedMapUKDGSnapshot"
+                elif "SnomedCT_UKEditionRF2_PRODUCTION" in paths[i]:
+                    opcs4_ref_set = "der2_iisssciRefset_ExtendedMapUKEDSnapshot"
+                elif "SnomedCT_UKClinicalRefsetsRF2_PRODUCTION" in paths[i]:
+                    continue
+                else:
+                    pass
             for f in os.listdir(refset_terminology):
                 m = re.search(f'{opcs4_ref_set}'+r'_(.*)_\d*.txt', f)
                 if m:
