@@ -15,23 +15,26 @@ class CATTests(unittest.TestCase):
     def setUpClass(cls) -> None:
         cls.cdb = CDB.load(os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "examples", "cdb.dat"))
         cls.vocab = Vocab.load(os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "examples", "vocab.dat"))
-        cls.cdb.config.general["spacy_model"] = "en_core_web_md"
-        cls.cdb.config.ner['min_name_len'] = 2
-        cls.cdb.config.ner['upper_case_limit_len'] = 3
-        cls.cdb.config.general['spell_check'] = True
-        cls.cdb.config.linking['train_count_threshold'] = 10
-        cls.cdb.config.linking['similarity_threshold'] = 0.3
-        cls.cdb.config.linking['train'] = True
-        cls.cdb.config.linking['disamb_length_limit'] = 5
-        cls.cdb.config.general['full_unlink'] = True
+        cls.cdb.config.general.spacy_model = "en_core_web_md"
+        cls.cdb.config.ner.min_name_len = 2
+        cls.cdb.config.ner.upper_case_limit_len = 3
+        cls.cdb.config.general.spell_check = True
+        cls.cdb.config.linking.train_count_threshold = 10
+        cls.cdb.config.linking.similarity_threshold = 0.3
+        cls.cdb.config.linking.train = True
+        cls.cdb.config.linking.disamb_length_limit = 5
+        cls.cdb.config.general.full_unlink = True
         cls.undertest = CAT(cdb=cls.cdb, config=cls.cdb.config, vocab=cls.vocab)
+        cls._linkng_filters = cls.undertest.config.linking.filters.copy_of()
 
     @classmethod
     def tearDownClass(cls) -> None:
         cls.undertest.destroy_pipe()
 
     def tearDown(self) -> None:
-        self.cdb.config.annotation_output['include_text_in_output'] = False
+        self.cdb.config.annotation_output.include_text_in_output = False
+        # need to make sure linking filters are not retained beyond a test scope
+        self.undertest.config.linking.filters = self._linkng_filters.copy_of()
 
     def test_callable_with_single_text(self):
         text = "The dog is sitting outside the house."
@@ -132,7 +135,7 @@ class CATTests(unittest.TestCase):
                              checkpoint=checkpoint,
                              is_resumed=True)
         checkpoints = [f for f in os.listdir(ckpt_dir_path) if "checkpoint-" in f]
-        self.assertEqual(15, len(checkpoints))
+        self.assertEqual(13, len(checkpoints))
         self.assertTrue("checkpoint-%s-3" % ckpt_steps in checkpoints)
         self.assertTrue("checkpoint-%s-6" % ckpt_steps in checkpoints)
         self.assertTrue("checkpoint-%s-9" % ckpt_steps in checkpoints)
@@ -146,7 +149,6 @@ class CATTests(unittest.TestCase):
         self.assertTrue("checkpoint-%s-33" % ckpt_steps in checkpoints)
         self.assertTrue("checkpoint-%s-36" % ckpt_steps in checkpoints)
         self.assertTrue("checkpoint-%s-39" % ckpt_steps in checkpoints)
-        self.assertTrue("checkpoint-%s-40" % ckpt_steps in checkpoints)
 
     def test_resume_training_on_absent_checkpoints(self):
         ckpt_dir_path = tempfile.TemporaryDirectory().name
@@ -175,7 +177,7 @@ class CATTests(unittest.TestCase):
         self.assertFalse("text" in out)
 
     def test_get_entities_including_text(self):
-        self.cdb.config.annotation_output['include_text_in_output'] = True
+        self.cdb.config.annotation_output.include_text_in_output = True
         text = "The dog is sitting outside the house."
         out = self.undertest.get_entities(text)
         self.assertEqual({}, out["entities"])
@@ -191,7 +193,7 @@ class CATTests(unittest.TestCase):
         self.assertFalse("text" in out[2])
 
     def test_get_entities_multi_texts_including_text(self):
-        self.cdb.config.annotation_output['include_text_in_output'] = True
+        self.cdb.config.annotation_output.include_text_in_output = True
         in_data = [(1, "The dog is sitting outside the house."), (2, ""), (3, None)]
         out = self.undertest.get_entities_multi_texts(in_data, n_process=2)
         self.assertEqual(3, len(out))
@@ -248,6 +250,40 @@ class CATTests(unittest.TestCase):
         for step in range(1, (nepochs_train + nepochs_retrain) * num_of_documents):
             self.assertTrue(f"checkpoint-1-{step}" in checkpoints)
 
+    def test_train_supervised_does_not_retain_MCT_filters_default(self, extra_cui_filter=None):
+        data_path = os.path.join(os.path.dirname(__file__), "resources", "medcat_trainer_export_filtered.json")
+        before = str(self.undertest.config.linking.filters)
+        self.undertest.train_supervised(data_path, nepochs=1, use_filters=True, extra_cui_filter=extra_cui_filter)
+        after = str(self.undertest.config.linking.filters)
+        self.assertEqual(before, after)
+
+    def test_train_supervised_can_retain_MCT_filters(self, extra_cui_filter=None, retain_extra_cui_filter=False):
+        data_path = os.path.join(os.path.dirname(__file__), "resources", "medcat_trainer_export_filtered.json")
+        before = str(self.undertest.config.linking.filters)
+        self.undertest.train_supervised(data_path, nepochs=1, use_filters=True, retain_filters=True,
+                                        extra_cui_filter=extra_cui_filter, retain_extra_cui_filter=retain_extra_cui_filter)
+        after = str(self.undertest.config.linking.filters)
+        self.assertNotEqual(before, after)
+        with open(data_path, 'r') as f:
+            project0 = json.load(f)['projects'][0]
+        filtered_cuis = project0['cuis'].split(',')
+        if extra_cui_filter and retain_extra_cui_filter:
+            # in case of extra_cui_filter and its retention, only it is retained
+            filtered_cuis = extra_cui_filter
+        self.assertGreater(len(filtered_cuis), 0)
+        for filtered_cui in filtered_cuis:
+            with self.subTest(f'CUI: {filtered_cui}'):
+                self.assertTrue(filtered_cui in self.undertest.config.linking.filters.cuis)
+
+    def test_train_supervised_no_leak_extra_cui_filters(self):
+        self.test_train_supervised_does_not_retain_MCT_filters_default(extra_cui_filter={'C123', 'C111'})
+
+    def test_train_supervised_no_leak_extra_cui_filters_along_MCT_filters(self):
+        self.test_train_supervised_can_retain_MCT_filters(extra_cui_filter={'C0037284'})
+
+    def test_train_supervised_can_retain_extra_cui_filters_along_MCT_filters(self):
+        self.test_train_supervised_can_retain_MCT_filters(extra_cui_filter={'C0037284'}, retain_extra_cui_filter=True)
+
     def test_no_error_handling_on_none_input(self):
         out = self.undertest.get_entities(None)
         self.assertEqual({}, out["entities"])
@@ -279,7 +315,7 @@ class CATTests(unittest.TestCase):
         self.assertEqual([], out[2]["tokens"])
 
     def test_error_handling_multi_processes(self):
-        self.cdb.config.annotation_output['include_text_in_output'] = True
+        self.cdb.config.annotation_output.include_text_in_output = True
         out = self.undertest.get_entities_multi_texts([
                                            (1, "The dog is sitting outside the house 1."),
                                            (2, "The dog is sitting outside the house 2."),
@@ -312,18 +348,23 @@ class CATTests(unittest.TestCase):
         contents = [f for f in os.listdir(os.path.join(save_dir_path.name, full_model_pack_name))]
         self.assertTrue("cdb.dat" in contents)
         self.assertTrue("vocab.dat" in contents)
+        self.assertTrue("model_card.json" in contents)
+        with open(os.path.join(save_dir_path.name, full_model_pack_name, "model_card.json")) as file:
+            model_card = json.load(file)
+        self.assertTrue("MedCAT Version" in model_card)
 
     def test_load_model_pack(self):
         save_dir_path = tempfile.TemporaryDirectory()
         full_model_pack_name = self.undertest.create_model_pack(save_dir_path.name, model_pack_name="mp_name")
         cat = self.undertest.load_model_pack(os.path.join(save_dir_path.name, f"{full_model_pack_name}.zip"))
         self.assertTrue(isinstance(cat, CAT))
+        self.assertIsNotNone(cat.config.version.medcat_version)
 
     def test_hashing(self):
         save_dir_path = tempfile.TemporaryDirectory()
         full_model_pack_name = self.undertest.create_model_pack(save_dir_path.name, model_pack_name="mp_name")
         cat = self.undertest.load_model_pack(os.path.join(save_dir_path.name, f"{full_model_pack_name}.zip"))
-        self.assertEqual(cat.get_hash(), cat.config.version['id'])
+        self.assertEqual(cat.get_hash(), cat.config.version.id)
 
 
 if __name__ == '__main__':
