@@ -1,8 +1,10 @@
 import sys
-sys.path.insert(0, "/home/ubuntu/projects/MedCAT/")
+sys.path.insert(0, '/home/ubuntu/projects/MedCAT/')
 import os
 import json
 from django.shortcuts import render
+from django.http import StreamingHttpResponse, HttpResponse
+from wsgiref.util import FileWrapper
 from medcat.cat import CAT
 from medcat.cdb import CDB
 from medcat.utils.helpers import doc2html
@@ -13,34 +15,14 @@ from urllib.error import HTTPError
 from .models import *
 from .forms import DownloaderForm
 
-vocab_path = os.getenv('VOCAB_PATH', '/tmp/vocab.dat')
-cdb_path = os.getenv('CDB_PATH', '/tmp/cdb.dat')
-auth_callback_service = 'https://medcat.rosalind.kcl.ac.uk/auth_callback'
-validation_base_url = 'https://uts-ws.nlm.nih.gov/rest/isValidServiceValidate'
+AUTH_CALLBACK_SERVICE = 'https://medcat.rosalind.kcl.ac.uk/auth-callback'
+VALIDATION_BASE_URL = 'https://uts-ws.nlm.nih.gov/rest/isValidServiceValidate'
+VALIDATION_LOGIN_URL = f'https://uts.nlm.nih.gov/uts/login?service={AUTH_CALLBACK_SERVICE}'
 
-# TODO
-#neg_path = os.getenv('NEG_PATH', '/tmp/mc_negated')
+model_pack_path = os.getenv('MODEL_PACK_PATH', 'models/medmen_wstatus_2021_oct.zip')
 
 try:
-    if not os.path.exists(vocab_path):
-        vocab_url = os.getenv('VOCAB_URL')
-        urlretrieve(vocab_url, vocab_path)
-
-    if not os.path.exists(cdb_path):
-        cdb_url = os.getenv('CDB_URL')
-        print("*" * 399)
-        print(cdb_url)
-        urlretrieve(cdb_url, cdb_path)
-
-    vocab = Vocab.load(vocab_path)
-    cdb = CDB.load(cdb_path)
-    #    mc_negated = MetaCAT(save_dir=neg_path)
-    #    mc_negated.load()
-    #    cat = CAT(cdb=cdb, vocab=vocab, meta_cats=[mc_negated])
-    cat = CAT(cdb=cdb, vocab=vocab, config=cdb.config)
-    cat.spacy_cat.MIN_ACC = 0.30
-    cat.spacy_cat.MIN_ACC_TH = 0.30
-    cat.spacy_cat.ACC_ALWAYS = True
+    cat = CAT.load_model_pack(model_pack_path)
 except Exception as e:
     print(str(e))
 
@@ -55,11 +37,7 @@ def get_html_and_json(text):
                 new_ent['Pretty Name'] = ent[key]
             if key == 'icd10':
                 icd10 = ent.get('icd10', [])
-                new_ent['ICD-10 Code'] = icd10[-1]['chapter'] if icd10 else '-'
-            new_ent['OPCS Code'] = '-'
-            if key == 'opcs':
-                opcs = ent.get('opcs', [])
-                new_ent['OPCS Code'] = opcs[-1]['chapter'] if opcs else '-'
+                new_ent['ICD-10 Code'] = icd10[-1] if icd10 else '-'
             if key == 'cui':
                 new_ent['Identifier'] = ent[key]
             if key == 'types':
@@ -81,16 +59,14 @@ def get_html_and_json(text):
         a['annotations'][id] = new_ent
 
     doc_json = json.dumps(a)
-
-
     uploaded_text = UploadedText()
-    uploaded_text.text = str(text)
+    uploaded_text.text = len(str(text))#str(text) no saving of text anymore
     uploaded_text.save()
 
     return doc2html(doc), doc_json
 
 
-def train_annotations(request):
+def show_annotations(request):
     context = {}
     context['doc_json'] = '{"msg": "No documents yet"}'
 
@@ -105,17 +81,17 @@ def train_annotations(request):
 
 def validate_umls_user(request):
     ticket = request.GET.get('ticket', '')
-    validate_url = f'{validation_base_url}?service={auth_callback_service}&ticket={ticket}'
+    validate_url = f'{VALIDATION_BASE_URL}?service={AUTH_CALLBACK_SERVICE}&ticket={ticket}'
     try:
         is_valid = urlopen(validate_url, timeout=10).read().decode('utf-8')
         context = {
             'is_valid': is_valid == 'true'
         }
         if is_valid == 'true':
-            context["message"] = "License verified! Please fill in the following form before downloading model packs."
-            context["downloader_form"] = DownloaderForm()
+            context['message'] = 'License verified! Please fill in the following form before downloading models.'
+            context['downloader_form'] = DownloaderForm(MedcatModel.objects.all())
         else:
-            context["message"] = "License not found. Please request or renew your UMLS Metathesaurus License."
+            context['message'] = f'License not found. Please request or renew your UMLS Metathesaurus License. If you think you have got the license, try {VALIDATION_LOGIN_URL} again.'
     except HTTPError:
         context = {
             'is_valid': False,
@@ -125,14 +101,29 @@ def validate_umls_user(request):
         return render(request, 'umls_user_validation.html', context=context)
 
 
-def download(request):
-    if request.method == "POST":
-        downloader_form = DownloaderForm(request.POST)
+def download_model(request):
+    if request.method == 'POST':
+        downloader_form = DownloaderForm(MedcatModel.objects.all(), request.POST)
         if downloader_form.is_valid():
+            mp_name = downloader_form.cleaned_data['modelpack']
+            model = MedcatModel.objects.get(model_name=mp_name)
+            if model is not None:
+                mp_path = model.model_file.path
+            else:
+                return HttpResponse(f'Error: Unknown model "{downloader_form.modelpack}"')
+            resp = StreamingHttpResponse(FileWrapper(open(mp_path, 'rb')))
+            resp['Content-Type'] = 'application/zip'
+            resp['Content-Length'] = os.path.getsize(mp_path)
+            resp['Content-Disposition'] = f'attachment; filename={os.path.basename(mp_path)}'
+            downloader_form.instance.downloaded_file = os.path.basename(mp_path)
             downloader_form.save()
-            message = "Download url will be displayed..."
+            return resp
         else:
-            message = "Erorr: All non-optional fields must be filled out."
+            context = {
+                'is_valid': True,
+                'downloader_form': downloader_form,
+                'message': 'All non-optional fields must be filled out:'
+            }
+            return render(request, 'umls_user_validation.html', context=context)
     else:
-        message = "Erorr: Unknown HTTP method."
-    return render(request, 'download.html', context={"message": message})
\ No newline at end of file
+        return HttpResponse('Erorr: Unknown HTTP method.')
